/*
 * $Id: rb_edje.c 17 2004-06-22 19:37:54Z tilman $
 *
 * Copyright (C) 2004 Tilman Sauerbeck (tilman at code-monkey de)
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */

#include <ruby.h>

#include <Edje.h>
#include <rb_evas.h>
#include <rb_evas_object.h>

#include "rb_edje_main.h"

#define GET_OBJ(obj, type, o, desc) \
	type **(o) = NULL; \
\
	Data_Get_Struct ((obj), type *, (o)); \
\
	if (!*(o)) { \
		rb_raise (rb_eException, desc " destroyed already"); \
		return Qnil; \
	}

VALUE cEdje;

static VALUE c_new (VALUE klass, VALUE evas)
{
	VALUE self, argv[1];
	Evas_Object **edje;

	GET_OBJ (evas, Evas, e, "Evas");

	self = Data_Make_Struct (klass, Evas_Object *, c_evas_object_mark,
	                         c_evas_object_free, edje);
	*edje = edje_object_add (*e);

	argv[0] = evas;
	rb_obj_call_init (self, 1, argv);

	return self;
}

static VALUE c_load (VALUE self, VALUE eet, VALUE group)
{
	GET_OBJ (self, Evas_Object, e, "Edje");

	Check_Type (eet, T_STRING);
	Check_Type (group, T_STRING);

	if (!edje_object_file_set (*e, StringValuePtr (eet),
	                           StringValuePtr (group)))
		rb_raise (rb_eException, "Cannot load eet");

	return Qnil;
}

static VALUE c_get_size_min (VALUE self)
{
	int w = 0, h = 0;

	GET_OBJ (self, Evas_Object, e, "Edje");

	edje_object_size_min_get (*e, &w, &h);

	return rb_ary_new3 (2, INT2FIX (w), INT2FIX (h));
}

static VALUE c_get_size_max (VALUE self)
{
	int w = 0, h = 0;

	GET_OBJ (self, Evas_Object, e, "Edje");

	edje_object_size_max_get (*e, &w, &h);

	return rb_ary_new3 (2, INT2FIX (w), INT2FIX (h));
}

static VALUE c_part_exists_get (VALUE self, VALUE part)
{
	int r;

	GET_OBJ (self, Evas_Object, e, "Edje");

	Check_Type (part, T_STRING);

	r = edje_object_part_exists (*e, StringValuePtr (part));

	return r ? Qtrue : Qfalse;
}

static VALUE c_part_swallow (VALUE self, VALUE part, VALUE target)
{
	GET_OBJ (self, Evas_Object, e, "Edje");

	Check_Type (part, T_STRING);

	if (!rb_obj_is_kind_of (target, cEvasObject)) {
		rb_raise (rb_eTypeError,
		          "wrong argument type %s (expected EvasObject)",
		          rb_obj_classname (target));
		return Qnil;
	}

	GET_OBJ (target, Evas_Object, target2, "EvasObject");

	edje_object_part_swallow (*e, StringValuePtr (part), *target2);

	return Qnil;
}

static VALUE c_part_unswallow (VALUE self, VALUE target)
{
	GET_OBJ (self, Evas_Object, e, "Edje");

	if (!rb_obj_is_kind_of (target, cEvasObject)) {
		rb_raise (rb_eTypeError,
		          "wrong argument type %s (expected EvasObject)",
		          rb_obj_classname (target));
		return Qnil;
	}

	GET_OBJ (target, Evas_Object, target2, "EvasObject");

	edje_object_part_unswallow (*e, *target2);

	return Qnil;
}

static VALUE c_get_part_swallow (VALUE self, VALUE part)
{
	Evas_Object *o;
	void *obj;

	GET_OBJ (self, Evas_Object, e, "Edje");

	Check_Type (part, T_STRING);

	if (!(o = edje_object_part_swallow_get (*e, StringValuePtr (part))))
		return Qnil;

	if (!(obj = evas_object_data_get (o, RUBY_EVAS_OBJECT_KEY))) {
		rb_raise (rb_eException, "EvasObject Ruby object key missing");
		return Qnil;
	}

	return (VALUE) obj;
}

void Init_Edje (void)
{
	cEdje = rb_define_class_under (mEdje, "Edje", cEvasObject);

	rb_define_singleton_method (cEdje, "new", c_new, 1);
	rb_define_method (cEdje, "load", c_load, 2);
	rb_define_method (cEdje, "get_size_min", c_get_size_min, 0);
	rb_define_method (cEdje, "get_size_max", c_get_size_max, 0);
	rb_define_method (cEdje, "part_exists?", c_part_exists_get, 1);
	rb_define_method (cEdje, "part_swallow", c_part_swallow, 2);
	rb_define_method (cEdje, "part_unswallow", c_part_unswallow, 1);
	rb_define_method (cEdje, "get_part_swallow", c_get_part_swallow, 1);
}

